//****************************************************************************
//**                                                                         *
//**  FILE        :  template.lsl                                            *
//**                                                                         *
//**  DESCRIPTION :  Project specific Linker Script.                         *
//**                                                                         *
//**  Copyright 2007-2010 Altium BV                                         *
//**                                                                         *
//****************************************************************************

#define CSA 192
// Include the CPU specific .lsl file
// The CPU is specified by the __CPU__ macro
#ifndef __CPU__
# error No CPU defined, please link with -D__CPU__=<cpu>
#endif

#define __QUOTE_ME2(x) #x
#define __QUOTE_ME(x) __QUOTE_ME2(x)
#define CPULSL_FILE __QUOTE_ME(__CPU__.lsl)

#include CPULSL_FILE

// Define default external memory
// These memory definitions can be replaced by the Debug Target Configuration Wizard

#ifndef HAS_ON_CHIP_FLASH       // On-chip flash is located at 0x[8a]0000000
#endif

#ifndef HAS_NO_EXTERNAL_RAM     // No EBU or no xRam
#endif

#ifndef LIBRARIES       /* libraries in abs24 space, but reserve abs18 */
#define LIBRARIES       [0x00000000+16k .. 0x00000000+2M] | \
                        [0x10000000+16k .. 0x10000000+2M] | \
                        [0x20000000+16k .. 0x20000000+2M] | \
                        [0x30000000+16k .. 0x30000000+2M] | \
                        [0x40000000+16k .. 0x40000000+2M] | \
                        [0x50000000+16k .. 0x50000000+2M] | \
                        [0x60000000+16k .. 0x60000000+2M] | \
                        [0x70000000+16k .. 0x70000000+2M] | \
                        [0x80000000+16k .. 0x80000000+2M] | \
                        [0x90000000+16k .. 0x90000000+2M] | \
                        [0xa0000000+16k .. 0xa0000000+2M] | \
                        [0xb0000000+16k .. 0xb0000000+2M] | \
                        [0xc0000000+16k .. 0xc0000000+2M] | \
                        [0xd0000000+16k .. 0xd0000000+2M] | \
                        [0xe0000000+16k .. 0xe0000000+2M] | \
                        [0xf0000000+16k .. 0xf0000000+2M]
#endif

section_layout ::linear
{
        // libraries in abs24 space, but reserve abs18 
        // space for .zro variables 
        group libraries (ordered, contiguous, run_addr=LIBRARIES)
        {
                select ".text*.libc";
                select ".text*.libcs";
                select ".text*.libcs_fpu";
                select ".text*.libfp";
                select ".text*.libfpt";
                select ".text*.librt";
                select ".text*.libcp*";
                select ".text*.libstl*";
        }
}




section_layout ::linear
{
    group stack "ustack"( size = 1k );
}
section_layout ::linear
{
    group heap "heap"( size = 32k );
}
section_layout ::linear ( direction = low_to_high )
{
    group privileged_ro ( ordered, contiguous, run_addr = mem:spe:pflash )
    {
        select "privileged_functions" ;
    }
    group privileged_rw ( ordered, contiguous, run_addr = mem:xram_a )
    {
        select "privileged_data" ;
    }
	"_lc_mb_rom" = addressof(mem:spe:pflash);
	"_lc_me_rom" = addressof(mem:spe:pflash) + sizeof(mem:spe:pflash);
	"_lc_mb_ram" = addressof(mem:xram_a);
	"_lc_me_ram" = addressof(mem:xram_a) + sizeof(mem:xram_a);
}







memory xrom_a (tag="dtc")
{
    mau = 8;
    size = 512k;
    type = rom;
    map (dest=bus:spe:fpi_bus, dest_offset=0xa1000000, size=512k);
}
memory xram_a (tag="dtc")
{
    mau = 8;
    size = 512k;
    type = ram;
    map (dest=bus:spe:fpi_bus, dest_offset=0xa1080000, size=512k);
}
memory flash0 (tag="flash=AM29BL162C")
{
    mau  = 8;
    type = rom;
    size = 2M*2;
    map( dest=bus:spe:fpi_bus, src_offset = 0x0, dest_offset = 0xa2000000, size = 2M*2 );
}
