/*
    FreeRTOS V6.0.5 - Copyright (C) 2010 Real Time Engineers Ltd.

    ***************************************************************************
    *                                                                         *
    * If you are:                                                             *
    *                                                                         *
    *    + New to FreeRTOS,                                                   *
    *    + Wanting to learn FreeRTOS or multitasking in general quickly       *
    *    + Looking for basic training,                                        *
    *    + Wanting to improve your FreeRTOS skills and productivity           *
    *                                                                         *
    * then take a look at the FreeRTOS eBook                                  *
    *                                                                         *
    *        "Using the FreeRTOS Real Time Kernel - a Practical Guide"        *
    *                  http://www.FreeRTOS.org/Documentation                  *
    *                                                                         *
    * A pdf reference manual is also available.  Both are usually delivered   *
    * to your inbox within 20 minutes to two hours when purchased between 8am *
    * and 8pm GMT (although please allow up to 24 hours in case of            *
    * exceptional circumstances).  Thank you for your support!                *
    *                                                                         *
    ***************************************************************************

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public
    License and the FreeRTOS license exception along with FreeRTOS; if not it
    can be viewed here: http://www.freertos.org/a00114.html and also obtained
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

#ifndef PORTMACRO_H
#define PORTMACRO_H

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------
 * Port specific definitions.
 *
 * The settings in this file configure FreeRTOS correctly for the
 * given hardware and compiler.
 *
 * These settings should not be altered.
 *-----------------------------------------------------------
 */

/* Type definitions. */
#define portCHAR		char
#define portFLOAT		float
#define portDOUBLE		double
#define portLONG		long
#define portSHORT		short
#define portSTACK_TYPE	unsigned long
#define portBASE_TYPE	long

#if( configUSE_16_BIT_TICKS == 1 )
	typedef unsigned portSHORT portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffff
#else
	typedef unsigned portLONG portTickType;
	#define portMAX_DELAY ( portTickType ) 0xffffffff
#endif
/*---------------------------------------------------------------------------*/

/* Architecture specifics. */
#define portSTACK_GROWTH							( -1 )
#define portTICK_RATE_MS							( ( portTickType ) 1000 / configTICK_RATE_HZ )
#define portBYTE_ALIGNMENT							4
#define portNOP()									__asm volatile( " nop " )
#define portCRITICAL_NESTING_IN_TCB					1
#define portRESTORE_FIRST_TASK_PRIORITY_LEVEL		1
#define portKERNEL_INTERRUPT_PRIORITY_LEVEL			8
#define portSYSTEM_INTERRUPT_PRIORITY_LEVEL			64

#if	configUSE_MPU == 1
	#define portUSING_MPU_WRAPPERS					1
	#define portREMOVE_STATIC_QUALIFIER				1
#endif
/*---------------------------------------------------------------------------*/

#if portUSING_MPU_WRAPPERS == 1

#include "mpu_wrappers.h"

/* MPU Specific Settings. */
#define portPRIVILEGE_BIT							0x80000000UL

#define portNUM_CONFIGURABLE_REGIONS					5
#define portTOTAL_NUMBER_DATA_REGIONS					4
#define portTOTAL_NUMBER_CODE_REGIONS					2

typedef struct MPU_REGION_REGISTERS
{
	unsigned portLONG ulRegionLowerAddress;
	unsigned portLONG ulRegionUpperAddress;
} xMPU_REGION_REGISTERS;

typedef struct MPU_SETTINGS
{
	xMPU_REGION_REGISTERS xDataRegions[ portTOTAL_NUMBER_DATA_REGIONS ];
	xMPU_REGION_REGISTERS xCodeRegions[ portTOTAL_NUMBER_CODE_REGIONS ];
	unsigned portBASE_TYPE uxDataRegionAttributes;
	unsigned portBASE_TYPE uxCodeRegionAttributes;
} xMPU_SETTINGS;

extern void vPortSetMPURegisterSetOne( xMPU_SETTINGS *pxMPUSettings );
extern portBASE_TYPE xPortRaisePrivilege( void );

#define portMPU_DATA_READ_WRITE_ACCESS			( 0xC0 )
#define portMPU_DATA_READ_ACCESS				( 0x40 )
#define portMPU_DATA_WRITE_ACCESS				( 0x80 )
#define portMPU_DATA_NO_ACCESS					( 0x00 )
#define portMPU_CODE_EXECUTE_ACCESS				( 0x80 )
#define portMPU_CODE_NO_EXECUTE					( 0x00 )

#define portMPU_SET_DATA_REGION_ATTRIBUTES( xRegion, xAttributes )		( ( xAttributes ) << ( ( xRegion ) * 8 ) )
#define portMPU_SET_CODE_REGION_ATTRIBUTES( xRegion, xAttributes )		( ( xAttributes ) << ( ( xRegion ) * 8 ) )

extern void MPU_vTaskEnterCritical( void );
extern void MPU_vTaskExitCritical( void );
#define portENTER_CRITICAL()			MPU_vTaskEnterCritical()
#define portEXIT_CRITICAL()				MPU_vTaskExitCritical()
/*---------------------------------------------------------------------------*/

extern __far void _lc_ub_heap[];		/* Heap */
extern __far void _lc_ue_heap[];		/* Heap end */
extern __far void _lc_ub_ustack[];		/* user stack */
extern __far void _lc_ue_ustack[];		/* user stack end */
extern __far void _lc_ub_istack[];		/* interrupt stack */
extern __far void _lc_ue_istack[];		/* interrupt stack end */
extern __far void _lc_gb_int_tab[];		/* interrupt table */
extern __far void _lc_ge_trap_tab[];	/* trap table */
extern int _lc_ub_csa_01[];				/* context save area 1 begin */
extern int _lc_ue_csa_01[];				/* context save area 1 end   */

extern unsigned long _lc_gb_privileged_ro[];		/* Privileged Read-only begin including functions. */
extern unsigned long _lc_ge_privileged_ro[];		/* Privileged Read-only end including functions. */
extern unsigned long _lc_gb_privileged_rw[];		/* Privileged Read/Write begin */
extern unsigned long _lc_ge_privileged_rw[];		/* Privileged Read/Write begin */
extern unsigned long _lc_mb_ram[];					/* RAM Start */
extern unsigned long _lc_me_ram[];					/* RAM End */
extern unsigned long _lc_mb_rom[];					/* ROM Start */
extern unsigned long _lc_me_rom[];					/* ROM End */

#define portHEAP_START_ADDRESS				( _lc_ub_heap )
#define portHEAP_STOP_ADDRESS				( _lc_ue_heap )
#define portMAIN_STACK_START_ADDRESS		( _lc_ub_ustack )
#define portMAIN_STACK_STOP_ADDRESS			( _lc_ue_ustack )
#define portINTERRUPT_STACK_START_ADDRESS	( _lc_ub_istack )
#define portINTERRUPT_STACK_STOP_ADDRESS	( _lc_ue_istack )
#define portINTERRUPT_TABLE_START_ADDRESS	( _lc_gb_int_tab )
#define portINTERRUPT_TABLE_STOP_ADDRESS	( _lc_ge_trap_tab )
#define portCONTEXT_SAVE_AREA_START_ADDRESS	( _lc_ub_csa_01 )
#define portCONTEXT_SAVE_AREA_STOP_ADDRESS	( _lc_ue_csa_01 )

#define portPRIVILEGED_ROM_START_ADDRESS	( _lc_gb_privileged_ro )
#define portPRIVILEGED_ROM_STOP_ADDRESS		( _lc_ge_privileged_ro )
#define portPRIVILEGED_RAM_START_ADDRESS	( _lc_gb_privileged_rw )
#define portPRIVILEGED_RAM_STOP_ADDRESS		( _lc_ge_privileged_rw )
#define portUNPRIVILEGED_ROM_START_ADDRESS	( _lc_ge_privileged_ro )
#define portUNPRIVILEGED_ROM_STOP_ADDRESS	( _lc_me_rom )
#define portUNPRIVILEGED_RAM_START_ADDRESS	( _lc_ge_privileged_rw )
#define portUNPRIVILEGED_RAM_STOP_ADDRESS	( _lc_ub_istack )

#define portRAM_START_ADDRESS				( _lc_mb_ram )
#define portRAM_STOP_ADDRESS				( _lc_me_ram )
#define portROM_START_ADDRESS				( _lc_mb_rom )
#define portROM_STOP_ADDRESS				( _lc_me_rom )
/*---------------------------------------------------------------------------*/

#else

typedef struct MPU_SETTINGS { unsigned long ulNotUsed; } xMPU_SETTINGS;

#define vPortSetKernelMPURegisterSet()
/* Define away the instruction from the Restore Context Macro. */
#define vPortSetMPURegisterSetOne( a )				( ( void ) ( a ) )
#define portPRIVILEGE_BIT							0x0UL

extern void vTaskEnterCritical( void );
extern void vTaskExitCritical( void );
#define portENTER_CRITICAL()			vTaskEnterCritical()
#define portEXIT_CRITICAL()				vTaskExitCritical()
/*---------------------------------------------------------------------------*/

#endif /* portUSING_MPU_WRAPPERS */

/*---------------------------------------------------------------------------*/

/* Task utilities. */

extern void vPortReclaimCSA( unsigned portBASE_TYPE *pxTCB );

/* CSA Manipulation. */
#define portCSA_TO_ADDRESS( pCSA )			( ( unsigned portBASE_TYPE * )( ( ( ( pCSA ) & 0x000F0000 ) << 12 ) | ( ( ( pCSA ) & 0x0000FFFF ) << 6 ) ) )
#define portADDRESS_TO_CSA( pAddress )		( ( unsigned portBASE_TYPE )( ( ( ( (unsigned portBASE_TYPE)( pAddress ) ) & 0xF0000000 ) >> 12 ) | ( ( (unsigned portBASE_TYPE)( pAddress ) & 0x003FFFC0 ) >> 6 ) ) )
/*---------------------------------------------------------------------------*/

#define portYIELD()								__asm volatile( " syscall #0 ")
/* Port Restore is implicit in the platform when the function is returned from the original PSW is automatically replaced. */
#define portMPU_RESTORE_PRIORITY( xPriority )	( ( void ) ( xPriority ) )
#define portSYSCALL_TASK_YIELD					0
#define portSYSCALL_RAISE_PRIORITY				1
/*---------------------------------------------------------------------------*/

/* Critical section management. */
/* Clear the ICR.IE bit. */		/* Or set ICR.CCPN to portSYSTEM_INTERRUPT_PRIORITY_LEVEL */
#define portDISABLE_INTERRUPTS()			__asm volatile( " disable \n" \
															" nop " );
/* Set the ICR.IE bit. */		/* Or set ICR.CCPN to 0 */
#define portENABLE_INTERRUPTS()				__asm volatile( " enable \n" \
															" nop " );
/*---------------------------------------------------------------------------*/

#define portYIELD_FROM_ISR( xHigherPriorityTaskWoken ) if( xHigherPriorityTaskWoken != pdFALSE ) vTaskSwitchContext()
/*---------------------------------------------------------------------------*/

/*
 * Save the context of a task.
 * The upper context is automatically saved when entering a trap or interrupt.
 * Need to save the lower context as well but first copy the FCX CSA ID into
 * pxCurrentTCB->pxTopOfStack. Issuing the BISR instruction consumes FCX to
 * save the lower context and re-enables the interrupts with a priority mask
 * above the System Interrupt Level.
 */
#define portSAVE_CONTEXT()													\
	__asm volatile( 														\
		" movh.a a15, #@his(pxCurrentTCB)		;Load pxCurrentTCB.\n" 		\
		" ld.a	a15, [a15]@los(pxCurrentTCB)	\n" 						\
		" mfcr	d14, #FCX						;Look at what PCX will be.\n" \
		" st.w	[a15], d14						;Save PCX for the Lower Context into pxTopOfStack. \n"	\
		" bisr	#64 							;Save the Lower Context and re-enables interrupts above System Interrupt Level.\n" \
		:																	\
		: 																	\
		: "d14", "a15"							);

/*---------------------------------------------------------------------------*/

/*
 * Restore the context of a task.
 * Disable Interrupts. Need to restore the lower context by loading the CSA from
 * pxCurrentTCB->pxTopOfStack into PCXI (effectively changing the call stack).
 * RSLCX will restore the lower context of the task. RFE will restore the upper
 * context of the task, jump to the return address and restore the previous
 * state of interrupts being enabled/disabled.
 */
#define portRESTORE_CONTEXT()	{ 											\
	portDISABLE_INTERRUPTS();												\
	vPortSetMPURegisterSetOne(((xMPU_SETTINGS *)&(pxCurrentTCB[ 1 ])));		\
	__asm volatile( 														\
		" ld.w	d14, [%0]				;Load PCXI from pxCurrentTCB->pxTopOfStack\n"	\
		" mtcr 	#PCXI, d14				;Set PCXI\n"						\
		" isync							;Settle\n" 							\
		" rslcx							;Restore the Lower Context.\n"		\
		" nop							;Settle\n"							\
		" rfe							;Return from Exception to restore Upper Context. RFE\n" \
		:																	\
		: "a"(pxCurrentTCB)													\
		: "d14"							); }
/*---------------------------------------------------------------------------*/

/* Task function macros as described on the FreeRTOS.org WEB site. */
#define portTASK_FUNCTION_PROTO( vFunction, pvParameters ) void vFunction( void *pvParameters )
#define portTASK_FUNCTION( vFunction, pvParameters ) void vFunction( void *pvParameters )
/*---------------------------------------------------------------------------*/

#ifdef __cplusplus
}
#endif

#endif /* PORTMACRO_H */
